import os
from ttp import ttp
from typing import Optional, List, Dict

def get_template(
        platform: Optional[str] = None, 
        command: Optional[str] = None, 
        yang: Optional[str] = None,
        misc: Optional[str] = None,
        path: Optional[str] = None
    ):
    """
    Function to locate template file and return it's content
    
    **Attributes**
    
    * path (str) - OS path to template to load
    * platform (str) - name of the platform to load template for
    * command (str) - command to load template for
    * yang (str) - name of YANG module to load template for
    * misc (str) - OS path to template within repository misc folder    
    
    **Valid combinations of template location**
    
    ``path`` attribute is always more preferred
    
    * ``path="./misc/foo/bar.txt"`` 
    * ``platfrom="cisco_ios", command="show version"``
    * ``yang="ietf-interfaces", platform="cisco_ios"``
    * ``misc="foo_folder/bar_template.txt"``       
    """
    # form path to template file
    if path:
        pass
    elif platform and command:
        platform = platform.lower()
        command = command.lower()
        command = command.replace("|", "pipe")
        for symbol in [" "]:
            platform = platform.replace(symbol, "_")
            command = command.replace(symbol, "_")
        path = "platform/{}_{}.txt".format(platform, command)
    elif platform and yang:
        platform = platform.lower()
        yang = yang.lower()
        for symbol in [" "]:
            platform = platform.replace(symbol, "_")
            yang = yang.replace(symbol, "_")        
        path = "yang/{}_{}.txt".format(yang, platform)
    elif misc:
        path = "misc/{}".format(misc)
    else:
        return None

    template_filename = os.path.join(os.path.dirname(__file__), path)
    
    # open template file and return content
    with open(template_filename, "r") as f:
        return f.read()
        

def parse_output(
        data: str, 
        platform: Optional[str] = None, 
        command: Optional[str] = None, 
        path: Optional[str] = None,
        yang: Optional[str] = None,
        misc: Optional[str] = None,
        structure: Optional[str] = "list",
        template_vars: Optional[Dict] = {}
    ):
    """
    Function to load template text and parse data provided
    
    **Attributes**
    
    * data (str) - data to parse
    * path (str) - OS path to template to load
    * platform (str) - name of the platform to load template for
    * command (str) - command to load template for
    * yang (str) - name of YANG module to load template for
    * misc (str) - OS path to template within repository misc folder    
    * structure (str) - results structure list, dictionary or flat_list
    * template_vars (dict) - variables to load in template object
    
    **Valid combinations of template location**
    
    ``path`` attribute is always more preferred
    
    * ``path="./misc/foo/bar.txt"`` 
    * ``platfrom="cisco_ios", command="show version"``
    * ``yang="ietf-interfaces", platform="cisco_ios"``
    * ``misc="foo_folder/bar_template.txt"``           
    """
    # get template text
    template = get_template(
        platform=platform, 
        command=command, 
        path=path,
        yang=yang,
        misc=misc
    )
    
    # create parser object
    parser = ttp(data=data, template=template, vars=template_vars)
        
    # parse and return results
    parser.parse(one=True)
    return parser.result(structure=structure)


import sys
import pprint
sys.path.insert(0,'..')

from ttp_templates import get_template
from ttp_templates import parse_output


def test_get_template():
    template = get_template(path="yang/ietf-interfaces_cisco_ios.txt")
    # print(template)
    assert isinstance(template, str)

# test_get_template()

def test_parse_output_platform():
    data = """
interface GigabitEthernet1/3.251
 description Customer #32148
 encapsulation dot1q 251
 ip address 172.16.33.10 255.255.255.128
 shutdown
!
interface GigabitEthernet1/3.251
 description Customer #32148
 encapsulation dot1q 251
 ip address 172.16.33.10 255.255.255.128
 shutdown
    """
    result = parse_output(
        data=data,
        platform="Test Platform",
        command="show run | sec interface"
    )
    # pprint.pprint(result)
    assert result == [[[{'description': 'Customer #32148',
                         'disabled': True,
                         'dot1q': '251',
                         'interface': 'GigabitEthernet1/3.251',
                         'ip': '172.16.33.10',
                         'mask': '255.255.255.128'},
                        {'description': 'Customer #32148',
                         'disabled': True,
                         'dot1q': '251',
                         'interface': 'GigabitEthernet1/3.251',
                         'ip': '172.16.33.10',
                         'mask': '255.255.255.128'}]]]
    
# test_parse_output()

def test_parse_output_misc():
    data = """
r1# show run | sec interfaces
interface GigabitEthernet1
 vrf forwarding MGMT
 ip address 10.223.89.55 255.255.255.0
 negotiation auto
 no mop enabled
interface GigabitEthernet1
 vrf forwarding MGMT
 ip address 10.223.89.56 255.255.255.0
 negotiation auto
 no mop enabled
 no mop sysid
    """
    result = parse_output(
        data=data,
        misc="ttp_templates_tests/cisco_ios_interfaces_cfg_per_ip.txt"
    )
    # pprint.pprint(result)
    assert result == [[{'description': 'description',
                        'hostname': 'r1',
                        'interface': 'GigabitEthernet1',
                        'ip': '10.223.89.55',
                        'mask': '255.255.255.0',
                        'vrf': 'MGMT'},
                       {'description': 'description',
                        'hostname': 'r1',
                        'interface': 'GigabitEthernet1',
                        'ip': '10.223.89.56',
                        'mask': '255.255.255.0',
                        'vrf': 'MGMT'}]]
   
   
# test_parse_output_misc()


import sys
import pprint
sys.path.insert(0,'..')

from ttp_templates import get_template
from ttp import ttp


def test_cisco_ios():
    data1 = """
interface GigabitEthernet1/3.251
 description Customer #32148
 encapsulation dot1q 251
 ip address 172.16.33.10 255.255.255.128
 shutdown
!
interface GigabitEthernet1/4
 description vCPEs access control
 ip address 172.16.33.10 255.255.255.128
!
    """
    data2 = """
interface GigabitEthernet1/5
 description Works data
 ip mtu 9000
!
interface GigabitEthernet1/7
 description Works data v6
 ipv6 address 2001::1/64
 ipv6 address 2001:1::1/64    
    """
    template = get_template(yang="ietf-interfaces", platform="cisco_ios")
    # print(template)
    parser = ttp(template=template)
    parser.add_input(data1)
    parser.add_input(data2)
    parser.parse()
    res = parser.result()
    # pprint.pprint(res)
    assert res == [{'comment': '',
                    'exception': {},
                    'result': [{'ietf-interfaces:interfaces': {'interface': [{'admin-status': 'down',
                                                                              'description': 'Customer '
                                                                                             '#32148',
                                                                              'enabled': False,
                                                                              'ietf-ip:ipv4': {'address': [{'ip': '172.16.33.10',
                                                                                                            'netmask': '255.255.255.128',
                                                                                                            'origin': 'static'}]},
                                                                              'if-index': 1,
                                                                              'link-up-down-trap-enable': 'enabled',
                                                                              'name': 'GigabitEthernet1/3.251',
                                                                              'oper-status': 'unknown',
                                                                              'statistics': {'discontinuity-time': '1970-01-01T00:00:00+00:00'},
                                                                              'type': 'iana-if-type:ethernetCsmacd'},
                                                                             {'admin-status': 'up',
                                                                              'description': 'vCPEs '
                                                                                             'access '
                                                                                             'control',
                                                                              'enabled': True,
                                                                              'ietf-ip:ipv4': {'address': [{'ip': '172.16.33.10',
                                                                                                            'netmask': '255.255.255.128',
                                                                                                            'origin': 'static'}]},
                                                                              'if-index': 1,
                                                                              'link-up-down-trap-enable': 'enabled',
                                                                              'name': 'GigabitEthernet1/4',
                                                                              'oper-status': 'unknown',
                                                                              'statistics': {'discontinuity-time': '1970-01-01T00:00:00+00:00'},
                                                                              'type': 'iana-if-type:ethernetCsmacd'}]}},
                               {'ietf-interfaces:interfaces': {'interface': [{'admin-status': 'up',
                                                                              'description': 'Works '
                                                                                             'data',
                                                                              'enabled': True,
                                                                              'ietf-ip:ipv4': {'mtu': 9000},
                                                                              'if-index': 1,
                                                                              'link-up-down-trap-enable': 'enabled',
                                                                              'name': 'GigabitEthernet1/5',
                                                                              'oper-status': 'unknown',
                                                                              'statistics': {'discontinuity-time': '1970-01-01T00:00:00+00:00'},
                                                                              'type': 'iana-if-type:ethernetCsmacd'},
                                                                             {'admin-status': 'up',
                                                                              'description': 'Works '
                                                                                             'data '
                                                                                             'v6',
                                                                              'enabled': True,
                                                                              'ietf-ip:ipv6': {'address': [{'ip': '2001::1',
                                                                                                            'origin': 'static',
                                                                                                            'prefix-length': 64},
                                                                                                           {'ip': '2001:1::1',
                                                                                                            'origin': 'static',
                                                                                                            'prefix-length': 64}]},
                                                                              'if-index': 1,
                                                                              'link-up-down-trap-enable': 'enabled',
                                                                              'name': 'GigabitEthernet1/7',
                                                                              'oper-status': 'unknown',
                                                                              'statistics': {'discontinuity-time': '1970-01-01T00:00:00+00:00'},
                                                                              'type': 'iana-if-type:ethernetCsmacd'}]}}],
                    'valid': {0: True, 1: True}}]
    
# test_cisco_ios()


from setuptools import setup, find_packages

with open("README.md", "r") as f:
    long_description = f.read()

__author__ = "Denis Mulyalin <d.mulyalin@gmail.com>"

setup(
    name="ttp_templates",
    version="0.1.1",
    author="Denis Mulyalin",
    author_email="d.mulyalin@gmail.com",
    description="Template Text Parser Templates collections",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/dmulyalin/ttp_templates",
    packages=find_packages(),
    extras_require={},
    include_package_data=True,
    package_data={
        "ttp_templates": [
            "platform/*.txt", 
            "yang/*.txt",
            "misc/*/*.txt"
            ]
    },
    data_files=[('', ['LICENSE'])],
    classifiers=[
        "Topic :: Utilities",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)
